/*
 * BnetDiagTool.cpp
 * ================
 * Copyright (C) Matthew J. W.
 * All rights reserved.
 * Version 1.0
 *
 *
 * Abstract:
 *   This module contains the overall functionality of the Battle.net Diagnostic-
 *   Tool application.
 *
 *
 *******************************************************************************************************************************************************/
#include "BnetDiagTool.h"



////////////////////////////////////////////////////////////////////////////
//
// Variables
//
////////////////////////////////////////////////////////////////////////////
HANDLE hConsoleOut;           // Handle to the console output system.
HANDLE hConsoleIn;            // Handle to the console input system.
HANDLE hLog;                  // Handle to the application log file.
Dword  dwIoResult;            // Buffer for the result of I/O procedures.
Char   strInt[15];            // Used to store integer conversions during console and log I/O.
Char   strCommand[MAXCMDLEN]; // Stores commands entered by the user.



////////////////////////////////////////////////////////////////////////////
//
// Application Entry Point
//
////////////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------------
// Void main()
// ===========
//
// Description:
//   Application entry point. Continuously prompts the user for a command and
//   processes it.
//
//   The application will exit once an "exit" command has been entered.
//
//
// Parameters:
//   None.
//
//
// Return Value:
//   None.
//
//
Void main()
{
	//
	// Initialize the application.
	//
	if(Initialize())
	{
		//
		// Display initial application info.
		//
		ConsoleOut("-----------------------------------------------------------------------------\n");
		ConsoleOut("                         Battle.net Diagnostic Tool\n");
		ConsoleOut("                                  by Win32\n");
		ConsoleOut("-----------------------------------------------------------------------------\n");
		ConsoleOut("Type ? for a listing of available commands.\n\n");


		//
		// Continuously prompt the user for a command, until he/she desires to exit.
		//
		for(;;)
		{
			ZeroMemory(strCommand, MAXCMDLEN * sizeof(Char));


			//
			// Request the user enter a command.
			//
			ConsoleOut(MSG_PROMPT_PREFIX);
			ConsoleIn(strCommand, MAXCMDLEN);


			//
			// Process the command.
			//
			if(ProcessCommand()) break;
			

			//
			// Move down a few lines between commands.
			//
			ConsoleOut(MSG_NEWLINE);
			ConsoleOut(MSG_NEWLINE);
		}


		//
		// Perform application cleanup.
		//
		Cleanup();
	}
}



////////////////////////////////////////////////////////////////////////////
//
// Functions
//
////////////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------------
// Bool Initialize()
// =================
//
// Description:
//   Prepares the application for use.
//
//
// Parameters:
//   None.
//
//
// Return Value:
//   If the function succeeds, the return value is non-zero. Elsewise, zero
//   is returned.
//
//
Bool __stdcall Initialize()
{
	WSADATA WinsockInfo;


	//
	// Open the application log.
	//
	if(OpenLog())
	{
		WriteLog(MSG_APPINIT);


		//
		// Prepare the console.
		//
		WriteLog(MSG_INIT_CONSOLE);

		if(OpenConsole())
		{
			//
			// Set the control handler for the console.
			//
			WriteLog(MSG_INIT_CTRL_HANDLER);

			if(SetConsoleCtrlHandler((PHANDLER_ROUTINE) &OnConsoleCtrl, TRUE))
			{
				register Int iResult;


				//
				// Initialize Winsock.
				//
				WriteLog(MSG_INIT_WINSOCK);
				iResult = WSAStartup(MAKEWORD(2, 2), &WinsockInfo);

				if(!iResult)
				{
					//
					// SUCCESS:
					//
					WriteLog(MSG_INIT_DONE);
					return TRUE;
				}

				else
				{
					//
					// ERROR:
					//
					SetLastError(WSAGetLastError());
				}
			}
		}


		//
		// Write the error to the application log.
		//
		WriteLog(MSG_ERROR);
		WriteLogInt(GetLastError());
	}

	else MessageBox(NULL, MSG_CRITICAL_ERROR, "CRITICAL ERROR", MB_OK);


	//
	// FAILURE:
	//
	return FALSE;
}



//--------------------------------------------------------------------------
// Void Cleanup()
// ==============
//
// Description:
//   Performs a cleanup of application resources.
//
//
// Parameters:
//   None.
//
//
// Return Value:
//   None.
//
//
Void __stdcall Cleanup()
{
	WriteLog(MSG_EXIT);


	//
	// Close console I/O handles.
	//
	CloseHandle(hConsoleOut);
	CloseHandle(hConsoleIn);


	//
	// Unload the Winsock library.
	//
	if(WSACleanup() == SOCKET_ERROR) WriteLog(MSG_WSCLEANUP_ERROR);


	//
	// Close application log.
	//
	WriteLog(MSG_APPLOG_CLOSE);
	CloseLog();
}



//--------------------------------------------------------------------------
// Bool OnCosoleCtrl(Dword)
// ========================
//
// Description:
//   Handles control commands received from the console subsystem.
//
//
// Parameters:
//   [Dword] [dwCtrlType] - Identifies the control command.
//
//
// Return Value:
//   Returns a non-zero value. This is only relevent to the subsystem.
//
//
Bool __stdcall OnConsoleCtrl(Dword dwCtrlType)
{
	//
	// We handle all control commands.
	//
	return TRUE;
}



//--------------------------------------------------------------------------
// Bool ProcessCommand()
// =====================
//
// Description:
//   Processes a command.
//
//
// Parameters:
//   None.
//
//
// Return Value:
//   If it is determined the command indicates the application is to terminate,
//   a non-zero value is returned. Elsewise, zero is returned.
//
//
// Remarks:
//   The command which is processed resides in 'strCommand'.
//
//   Valid commands are: "resolve", "connect", "?" and "exit".
//
//
Bool __fastcall ProcessCommand()
{
	Word wParameters;


	//
	// Count the number of parameters in the user's command.
	//
	wParameters = 0;

	for(Int i = 0; i < MAXCMDLEN; i++) 
	{
		if(strCommand[i] == ' ') 
		{
			wParameters++;


			//
			// Convert the characters preceeding the seperator into a string
			// of their own.
			//
			strCommand[i] = NULL;
		}


		//
		// Nullify any newline characters.
		//
		if(strCommand[i] == '\n' || strCommand[i] == '\r') strCommand[i] = NULL;
	}


	//
	// Respond to the command accordingly.
	//
	if(CompareStringNoCase(strCommand, COMMAND_RESOLVE))
	{
		//
		// Ensure the correct number of parameters has been given for this command.
		//
		if(wParameters == 1)
		{
			//
			// Resolve the parameter-provided hostname.
			//
			Resolve(&strCommand[GetStringLength(strCommand) + 1]);
		}

		else ConsoleOut(MSG_BAD_NUM_PARAMS);
	}


	else if(CompareStringNoCase(strCommand, COMMAND_CONNECT))
	{
		//
		// Ensure the correct number of parameters has been given for this command.
		//
		if(wParameters == 1)
		{
			//
			// Connect to the parameter-provided Battle.net server.
			//
			Connect(inet_addr(&strCommand[GetStringLength(strCommand) + 1]));
		}

		else ConsoleOut(MSG_BAD_NUM_PARAMS);
	}

	else if(CompareStringNoCase(strCommand, COMMAND_HELP))
	{
		//
		// Display a listing of available commands.
		//
		ConsoleOut("\nresolve <Hostname>  - Retrieves the IP addresses tied to the given hostname.\n");
		ConsoleOut("connect <Server IP> - Connects to a Battle.net server.\n");
		ConsoleOut("exit                - Exits the application.\n");
		ConsoleOut("?                   - Displays this information.\n");
	}
		
	else if(CompareStringNoCase(strCommand, COMMAND_EXIT))
	{
		//
		// The user does wish to exit.
		//
		return TRUE;
	}

	else
	{
		//
		// Invalid command.
		//
		ConsoleOut(MSG_BAD_COMMAND);
	}


	//
	// The user doesn't wish to exit.
	//
	return FALSE;
}

			

//--------------------------------------------------------------------------
// Bool Resolve(Char*)
// ===================
//
// Description:
//   Resolves a hostname to a listing of IP addresses.
//
//
// Parameters:
//   [Char*] [pstrHostname] - The hostname to resolve.
//
//
// Return Value:
//   If the function succeeds, the return value is non-zero. Elsewise, zero
//   is returned.
//
//
Bool __fastcall Resolve(const Char* pstrHostname)
{
	WriteLog(MSG_RESOLVE_START);
	WriteLog(pstrHostname);

	register Int iResult;
	ADDRINFO*    pHostInfo;


	//
	// Resolve the given hostname to a listing of IP addresses.
	//
	ConsoleOut(MSG_RESOLVE_BEGIN);
	WriteLog(MSG_RESOLVE_BEGIN);
	iResult = getaddrinfo(pstrHostname, NULL, NULL, &pHostInfo);

	if(!iResult)
	{
		WriteLog(MSG_RESOLVE_SUCCEEDED);

		Char     *pstrIp;
		ADDRINFO *p = pHostInfo;
		

		//
		// Display the resolved host information.
		//
		for(;p;)
		{
			pstrIp = inet_ntoa(((SOCKADDR_IN*) p->ai_addr)->sin_addr);

			ConsoleOut("\n");
			ConsoleOut(pstrIp);
			WriteLog(MSG_NEWLINE);
			WriteLog(pstrIp);
			
			p = p->ai_next;
		}


		//
		// Free the host information linked-list.
		//
		freeaddrinfo(pHostInfo);


		//
		// SUCCESS:
		//
		return TRUE;
	}

	else
	{
		//
		// ERROR:
		//
		SetLastError(iResult);
	}


	//
	// Display the error.
	//
	ConsoleOut(MSG_ERROR);
	ConsoleOutInt(GetLastError());

	WriteLog(MSG_ERROR);
	WriteLogInt(GetLastError());


	//
	// FAILURE:
	//
	return FALSE;
}



//--------------------------------------------------------------------------
// Bool Connect(Dword)
// ===================
//
// Description:
//   Establishes a useable connection with a Battle.net server.
//
//
// Parameters:
//   [Dword] [dwServer] - Identifies the server to connect to.
//
//
// Return Value:
//   If the function succeeds, the return value is non-zero. Elsewise, zero
//   is returned.
//
//
// Remarks:
//   A connection is only considered useable when the client is not banned by
//   the server.
//
//
// Technical Insight:
//   All Battle.net servers accept connections on a TCP-level, even if the client
//   is banned.
//
//   If a client is banned by the server, it's connection will be closed shortly
//   after it's acknowledgment.
//
//
Bool __fastcall Connect(Dword dwServer)
{
	WriteLog(MSG_CONNECT_START);
	WriteLog(inet_ntoa(*((in_addr*) &dwServer)));

	SOCKET      hBnetSock;
	SOCKADDR_IN ServerAddr;
	SOCKADDR_IN LocalAddr;


	ZeroMemory(&LocalAddr, sizeof(SOCKADDR_IN));
	LocalAddr.sin_family = AF_INET; // TCP/IP protocol family.

	ZeroMemory(&ServerAddr, sizeof(SOCKADDR_IN));
	ServerAddr.sin_family      = AF_INET;     // TCP/IP protocol family.
	ServerAddr.sin_port        = SERVER_PORT; // 6112
	ServerAddr.sin_addr.s_addr = dwServer;


	//
	// Create a socket which can be used to connect to the Battle.net server.
	//
	WriteLog(MSG_CONNECT_CREATE);
	hBnetSock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

	if(hBnetSock != INVALID_SOCKET)
	{
		//
		// Associate the socket with any available address.
		//
		WriteLog(MSG_CONNECT_BIND);
		if(!bind(hBnetSock, (const SOCKADDR*) &LocalAddr, sizeof(SOCKADDR_IN)))
		{
			Dword dwAsync = TRUE;


			//
			// Place the socket into asynchronous (non-blocking) mode.
			//
			WriteLog(MSG_CONNECT_ASYNC);
			if(!ioctlsocket(hBnetSock, FIONBIO, (u_long*) &dwAsync))
			{
				Int iResult;


				//
				// Initiate a connection with the server.
				//
				WriteLog(MSG_CONNECT_INITIATE);
				connect(hBnetSock, (const SOCKADDR*) &ServerAddr, sizeof(SOCKADDR_IN));
				iResult = WSAGetLastError();

				if(iResult == WSAEWOULDBLOCK)
				{
					WriteLog(MSG_CONNECT_BEGIN);
					ConsoleOut(MSG_CONNECT_BEGIN);


					fd_set  Status;
					TIMEVAL Timeout;

					*Status.fd_array = hBnetSock;
					Status.fd_count  = 1;
					Timeout.tv_sec   = 0;
					Timeout.tv_usec  = 0;


					//
					// Wait for a connection to be properly established.
					//
					for(;;)
					{
						//
						// Check if the connection has been established.
						//
						if(select(NULL, NULL, &Status, NULL, (const TIMEVAL*) &Timeout) == 1)
						{
							WriteLog(MSG_CONNECT_CONNECTED);
							register Dword dwTimeout = GetTickCount() + SERVER_RESPONSE_TIMEOUT;


							//
							// Check the connection remains valid for a short period of time,
							// which will ensure we're not IP banned.
							//
							for(;;)
							{
								//
								// See if the connection has been valid long enough for it to be
								// considered usable.
								//
								if(GetTickCount() == dwTimeout)
								{
									WriteLog(MSG_CONNECT_DONE);
									ConsoleOut(MSG_CONNECT_DONE);


									//
									// Destroy the connection.
									//
									WriteLog(MSG_CONNECT_SHUTDOWN);
									if(!shutdown(hBnetSock, SD_BOTH))
									{
										WriteLog(MSG_CONNECT_CLOSE);
										if(!closesocket(hBnetSock))
										{
											//
											// SUCCESS:
											//
											return TRUE;
										}

										else
										{
											//
											// ERROR:
											//
											SetLastError(WSAGetLastError());

											break;
										}
									}

									else
									{
										//
										// ERROR:
										//
										SetLastError(WSAGetLastError());

										break;
									}
								}


								//
								// Test the connection.
								//
								recv(hBnetSock, (char*) &Timeout, 1, NULL);
								iResult = WSAGetLastError();

								if(iResult != WSAEWOULDBLOCK)
								{
									//
									// ERROR:
									//
									WriteLog(MSG_CONNECT_BANNED);
									ConsoleOut(MSG_CONNECT_BANNED);

									break;
								}


								//
								// Ensure we don't hog the CPU.
								//
								Sleep(1);
							}


							//
							// Shutdown the connection.
							//
							if(shutdown(hBnetSock, SD_BOTH)) WriteLog(MSG_ERROR_BAD_CLEANUP);

							break;;
						}

						Status.fd_count++;


						//
						// Check if any error occured.
						//
						if(select(NULL, NULL, NULL, &Status, (const TIMEVAL*) &Timeout) == 1)
						{
							Int iError;
							Int iErrorLen = sizeof(Int);


							//
							// ERROR:
							//
							if(!getsockopt(hBnetSock, SOL_SOCKET, SO_ERROR, (char*) &iError, (int*) &iErrorLen))
							{
								SetLastError(iError);
							}

							else
							{
								WriteLog(MSG_ERROR_FAILED);
								SetLastError(WSAGetLastError());
							}

							break;
						}

						Status.fd_count++;


						//
						// Ensure we don't hog the CPU.
						//
						Sleep(1);
					}
				}

				else
				{
					//
					// ERROR:
					//
					SetLastError(iResult);
				}
			}

			else
			{
				//
				// ERROR:
				//
				SetLastError(WSAGetLastError());
			}
		}

		else
		{
			//
			// ERROR:
			//
			SetLastError(WSAGetLastError());
		}


		//
		// Close down the socket.
		//
		if(closesocket(hBnetSock)) WriteLog(MSG_ERROR_BAD_CLEANUP);

		Dword w = WSAGetLastError();
		w = 0;
	}

	else
	{
		//
		// ERROR:
		//
		SetLastError(WSAGetLastError());
	}

	
	//
	// Display the error.
	//
	ConsoleOut(MSG_ERROR);
	ConsoleOutInt(GetLastError());

	WriteLog(MSG_ERROR);
	WriteLogInt(GetLastError());


	//
	// FAILURE:
	//
	return FALSE;
}



////////////////////////////////////////////////////////////////////////////
//
// Console Functions
//
////////////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------------
// Bool OpenConsole()
// ==================
//
// Description:
//   Prepares the console for use.
//
//
// Parameters:
//   None.
//
//
// Return Value:
//   If the function succeeds, the return value is non-zero. Elsewise, zero
//   is returned.
//
//
Bool __stdcall OpenConsole()
{
	//
	// Retrieve handles to console I/O system.
	//
	hConsoleOut = GetStdHandle(STD_OUTPUT_HANDLE);
	hConsoleIn  = GetStdHandle(STD_INPUT_HANDLE);

	if(hConsoleOut != INVALID_HANDLE_VALUE)
	{
		if(hConsoleIn != INVALID_HANDLE_VALUE)
		{
			//
			// SUCCESS:
			//
			return TRUE;
		}
	}


	//
	// FAILURE:
	//
	return FALSE;
}



//--------------------------------------------------------------------------
// Void ConsoleOut(Char*)
// ======================
//
// Description:
//   Prints a null-terminated string of characters to the console.
//
//
// Parameters:
//   pChar*] [pstr] - The string to be printed.
//
//
// Return Value:
//   None.
//
//
Void __fastcall ConsoleOut(const Char* pstr)
{
	//
	// Write the given string to the console.
	//
	WriteConsole(hConsoleOut, pstr, GetStringLength(pstr), (DWORD*) &dwIoResult, NULL);
}



//--------------------------------------------------------------------------
// Void ConsoleOutInt(Dword)
// =========================
//
// Description:
//   Prints an integer to the console.
//
//
// Parameters:
//   [Dword] [dwInteger] - The integer to be printed.
//
//
// Return Value:
//   None.
//
//
Void __fastcall ConsoleOutInt(Dword dwInteger)
{
	//
	// Convert the integer to a string.
	//
	ZeroMemory(strInt, 15);
	IntToStr(dwInteger, 10, strInt);


	//
	// Now output the integer to the console.
	//
	ConsoleOut(strInt);
}



//--------------------------------------------------------------------------
// Void ConsoleIn(Char*, Dword)
// ============================
//
// Description:
//   Prompts the user to give the console input, and copies it into a buffer
//   once the return key is entered.
//
//
// Parameters:
//   [Char*] [pstr]         - Pointer to a buffer which will receive the user input.
//   [Dword] [dwCharacters] - Size of the provided buffer, in bytes.
//
//
// Return Value:
//   None.
//
//
Void __fastcall ConsoleIn(Char* pstr, Dword dwCharacters)
{
	//
	// Read the requested number of characters from the console.
	//
	ReadConsole(hConsoleIn, pstr, dwCharacters, (DWORD*) &dwIoResult, NULL);
}



//--------------------------------------------------------------------------
// Void CloseConsole()
// ===================
//
// Description:
//   Disable the use of the console.
//
//
// Parameters:
//   None.
//
//
// Return Value:
//   None.
//
//
Void __stdcall CloseConsole()
{
	//
	// Close the console I/O handles.
	//
	CloseHandle(hConsoleOut);
	CloseHandle(hConsoleIn);
}



////////////////////////////////////////////////////////////////////////////
//
// Log Functions
//
////////////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------------
// Bool OpenLog()
// ==============
//
// Description:
//   Opens (creates) the application log-file, and prepares it for use.
//
//
// Parameters:
//   None.
//
//
// Return Value:
//   If the function succeeds, the return value is non-zero. Elsewise, zero
//   is returned.
//
//
Bool __fastcall OpenLog()
{
	//
	// Create a new log file.
	//
	hLog = CreateFile(LOG_FILENAME, GENERIC_WRITE, NULL, NULL, CREATE_ALWAYS, NULL, NULL);

	if(hLog != INVALID_HANDLE_VALUE)
	{
		//
		// Write initial log information.
		//
		PrepareLog();


		//
		// SUCCESS:
		//
		return TRUE;
	}

	
	//
	// FAILURE:
	//
	return FALSE;
}



//--------------------------------------------------------------------------
// Void WriteLog(Char*)
// ====================
//
// Description:
//   Writes a string of characters to the application log.
//
//
// Parameters:
//   [Char*] [pstr] - The string to be written.
//
//
// Return Value:
//   None.
//
//
Void __fastcall WriteLog(const Char* pstr)
{
	//
	// Write the given string to the log.
	//
	WriteFile(hLog, pstr, GetStringLength(pstr), (DWORD*) &dwIoResult, NULL);
}



//--------------------------------------------------------------------------
// Void WriteLog(Char)
// ===================
//
// Description:
//   Writes a single character to the application log.
//
//
// Parameters:
//   [Char] [c] - The character to be written.
//
//
// Return Value:
//   None.
//
//
Void __fastcall WriteLog(const Char c)
{
	//
	// Write the given character to the log.
	//
	WriteFile(hLog, &c, sizeof(Char), (DWORD*) &dwIoResult, NULL);
}



//--------------------------------------------------------------------------
// Void WriteLogInt(Dword)
// =======================
//
// Description:
//   Writes an integer to the application log.
//
//
// Parameters:
//   [Dword] [dwInteger] - The integer to be written.
//
//
// Return Value:
//   None.
//
//
Void __fastcall WriteLogInt(Dword dwInteger)
{
	//
	// Convert the integer to a string.
	//
	ZeroMemory(strInt, 15);
	IntToStr(dwInteger, 10, strInt);


	//
	// Now write the integer to the log.
	//
	WriteLog(strInt);
}



//--------------------------------------------------------------------------
// Void PrepareLog()
// =================
//
// Description:
//   Prepares the application log, writing initial system information to it.
//
//
// Parameters:
//   None.
//
//
// Return Value:
//   None.
//
//
Void __stdcall PrepareLog()
{
	Char             Buffer[1024];
	Dword            dwBuffSize;
    OSVERSIONINFOEXA VerInfo;
	MEMORYSTATUS     MemStatus;
	SYSTEM_INFO      SysInfo;
	HANDLE           hProcSnap;
	PROCESSENTRY32   ProcInfo;


	//
	// Log general system information.
	//
	WriteLog("------------------------------------------------------------------\r\n");
	WriteLog("                          System Overview                         \r\n");
	WriteLog("------------------------------------------------------------------\r\n");

	// Current system username.
	ZeroMemory(Buffer, 1024);
	dwBuffSize = 1024;
	if(GetUserName(Buffer, (DWORD*) &dwBuffSize))
	{
		WriteLog("User                      : ");
		WriteLog(Buffer);
		WriteLog(MSG_NEWLINE);
	}

	// Computer name.
	ZeroMemory(Buffer, 1024);
	dwBuffSize = 1024;
	if(GetComputerName(Buffer, (DWORD*) &dwBuffSize))
	{
		WriteLog("Computer Name             : ");
		WriteLog(Buffer);
		WriteLog(MSG_NEWLINE);
	}

	// System directory.
	ZeroMemory(Buffer, 1024);
	if(GetSystemDirectory(Buffer, 1024))
	{
		WriteLog("System Directory          : ");
		WriteLog(Buffer);
		WriteLog(MSG_NEWLINE);
	}

	// Windows directory.
	ZeroMemory(Buffer, 1024);
	if(GetSystemWindowsDirectory(Buffer, 1024))
	{
		WriteLog("Windows Directory         : ");
		WriteLog(Buffer);
		WriteLog(MSG_NEWLINE);
	}

	// Version info.
	VerInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEXA);

	if(GetVersionEx((OSVERSIONINFO*) &VerInfo))
	{
		// OS Version
		WriteLog("Operating System          : ");
		WriteLogInt(VerInfo.dwMajorVersion);
		WriteLog('.');
		WriteLogInt(VerInfo.dwMinorVersion);
		WriteLog(" (");
		if(VerInfo.dwMajorVersion == 4)                                     WriteLog("Windows NT 4.0");
		else if(VerInfo.dwMajorVersion == 6)                                WriteLog("Windows Vista");
		else if(VerInfo.dwMajorVersion == 5 && VerInfo.dwMinorVersion == 2) WriteLog("Windows Server 2003 or Windows XP Professional x64");
		else if(VerInfo.dwMajorVersion == 5 && VerInfo.dwMinorVersion == 1) WriteLog("Windows XP");
		else if(VerInfo.dwMajorVersion == 5 && VerInfo.dwMinorVersion == 0) WriteLog("Windows 2000");
		WriteLog(")\r\n");

		// OS Build
		WriteLog("OS Build                  : ");
		WriteLogInt(VerInfo.dwBuildNumber);
		WriteLog(MSG_NEWLINE);

		// Platform ID
		WriteLog("Platform ID               : ");
		WriteLogInt(VerInfo.dwPlatformId);
		WriteLog(MSG_NEWLINE);

		// Service pack.
		WriteLog("Service Pack              : ");
		WriteLog(VerInfo.szCSDVersion);
		WriteLog(MSG_NEWLINE);

		// Service pack version.
		WriteLog("Service Pack Version      : ");
		WriteLogInt(VerInfo.wServicePackMajor);
		WriteLog('.');
		WriteLogInt(VerInfo.wServicePackMinor);
		WriteLog(MSG_NEWLINE);

		// Product suites.
		WriteLog("Product Suites            : ");
		if(VerInfo.wSuiteMask & VER_SUITE_BACKOFFICE)               WriteLog("Microsoft BackOffice, ");
		if(VerInfo.wSuiteMask & VER_SUITE_BLADE)                    WriteLog("WinServer03 Web Edition, ");
		if(VerInfo.wSuiteMask & VER_SUITE_DATACENTER)               WriteLog("Data Center, ");
		if(VerInfo.wSuiteMask & VER_SUITE_ENTERPRISE)               WriteLog("Enterprise Edition, ");
		if(VerInfo.wSuiteMask & VER_SUITE_EMBEDDEDNT)               WriteLog("Windows XP Embedded, ");
		if(VerInfo.wSuiteMask & VER_SUITE_PERSONAL)                 WriteLog("Home Edition, ");
		if(VerInfo.wSuiteMask & VER_SUITE_SINGLEUSERTS)             WriteLog("Remote Desktop (Single), ");
		if(VerInfo.wSuiteMask & VER_SUITE_SMALLBUSINESS)            WriteLog("Microsoft Small Buisness, ");
		if(VerInfo.wSuiteMask & VER_SUITE_SMALLBUSINESS_RESTRICTED) WriteLog("Microsoft Small Buisness Server), ");
		if(VerInfo.wSuiteMask & VER_SUITE_TERMINAL)                 WriteLog("Terminal Services.");
		WriteLog(MSG_NEWLINE);

		// Product type.
		WriteLog("Platform Type             : ");
		if(VerInfo.wProductType == VER_NT_DOMAIN_CONTROLLER) WriteLog("Domain Controller");
		else if(VerInfo.wProductType == VER_NT_SERVER)       WriteLog("NT Server");
		else if(VerInfo.wProductType == VER_NT_WORKSTATION)  WriteLog("Workstation");
		WriteLog(MSG_NEWLINE);
	}

	// Memory status.
	GlobalMemoryStatus(&MemStatus);

	// Physical memory use %
	WriteLog("Physical Memory Usage     : ");
	WriteLogInt(MemStatus.dwMemoryLoad);
	WriteLog('%');
	WriteLog(MSG_NEWLINE);

	// Total physical memory.
	WriteLog("Total Physical Memory     : ");
	WriteLogInt(MemStatus.dwTotalPhys);
	WriteLog(MSG_NEWLINE);

	// Total virtual memory.
	WriteLog("Total Virtual Memory      : ");
	WriteLogInt(MemStatus.dwTotalVirtual);
	WriteLog(MSG_NEWLINE);

	// Page file size.
	WriteLog("Page File Size            : ");
	WriteLogInt(MemStatus.dwTotalPageFile);
	WriteLog(MSG_NEWLINE);

	// Available physical memory.
	WriteLog("Available Physical Memory : ");
	WriteLogInt(MemStatus.dwAvailPhys);
	WriteLog(MSG_NEWLINE);

	// Available virtual memory.
	WriteLog("Available Virtual Memory  : ");
	WriteLogInt(MemStatus.dwAvailVirtual);
	WriteLog(MSG_NEWLINE);

	// Available page file.
	WriteLog("Available Page File       : ");
	WriteLogInt(MemStatus.dwAvailPageFile);
	WriteLog(MSG_NEWLINE);


	//
	// Log processor information.
	//
	WriteLog("\r\n\r\n\r\n------------------------------------------------------------------\r\n");
	WriteLog("                       Processor Information                      \r\n");
	WriteLog("------------------------------------------------------------------\r\n");

	// System info.
	GetSystemInfo(&SysInfo);

	// Processor count.
	WriteLog("Processors             : ");
	WriteLogInt(SysInfo.dwNumberOfProcessors);
	WriteLog(MSG_NEWLINE);

	// Processor architecture.
	WriteLog("Processor Architecture : ");
	if(SysInfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64)              WriteLog("x64 Intel or AMD");
	else if(SysInfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_IA32_ON_WIN64) WriteLog("WOW64");
	else if(SysInfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_IA64)          WriteLog("Intel Itanium Processor Family");
	else if(SysInfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL)         WriteLog("x86 (Intel)");
	else																		    WriteLog("Unknown");
	WriteLog(MSG_NEWLINE);

	// Processor family.
	WriteLog("Processor Family       : ");
	if(SysInfo.dwProcessorType == PROCESSOR_INTEL_386)          WriteLog("386");
	else if(SysInfo.dwProcessorType == PROCESSOR_INTEL_486)     WriteLog("486");
	else if(SysInfo.dwProcessorType == PROCESSOR_INTEL_PENTIUM) WriteLog("Pentium");
	else if(SysInfo.dwProcessorType == PROCESSOR_INTEL_IA64)    WriteLog("IA64");
	else if(SysInfo.dwProcessorType == PROCESSOR_AMD_X8664)     WriteLog("AMD x86-64");

	WriteLog(MSG_NEWLINE);

	// Page size.
	WriteLog("Page Size              : ");
	WriteLogInt(SysInfo.dwPageSize);
	WriteLog(MSG_NEWLINE);

	// Min application memory address.
	WriteLog("Min. Memory Address    : ");
	WriteLogInt((Dword) SysInfo.lpMinimumApplicationAddress);
	WriteLog(MSG_NEWLINE);

	// Max application memory address.
	WriteLog("Max. Memory Address    : ");
	WriteLogInt((Dword) SysInfo.lpMaximumApplicationAddress);
	WriteLog(MSG_NEWLINE);

	// Processor features.
	
	// 3D Now!
	WriteLog("3D Now!                : ");
	if(IsProcessorFeaturePresent(PF_3DNOW_INSTRUCTIONS_AVAILABLE)) WriteLog(MSG_YES);
	else														   WriteLog(MSG_NO);
	WriteLog(MSG_NEWLINE);

	// MMX
	WriteLog("MMX                    : ");
	if(IsProcessorFeaturePresent(PF_MMX_INSTRUCTIONS_AVAILABLE)) WriteLog(MSG_YES);
	else														 WriteLog(MSG_NO);
	WriteLog(MSG_NEWLINE);

	// SSE3
	WriteLog("SSE3                   : ");
	if(IsProcessorFeaturePresent(13)) WriteLog(MSG_YES);
	else							  WriteLog(MSG_NO);
	WriteLog(MSG_NEWLINE);

	// SSE2
	WriteLog("SSE2                   : ");
	if(IsProcessorFeaturePresent(PF_XMMI64_INSTRUCTIONS_AVAILABLE)) WriteLog(MSG_YES);
	else														    WriteLog(MSG_NO);
	WriteLog(MSG_NEWLINE);

	// SSE
	WriteLog("SSE                    : ");
	if(IsProcessorFeaturePresent(6)) WriteLog(MSG_YES);
	else							 WriteLog(MSG_NO);
	WriteLog(MSG_NEWLINE);

	// Channels
	WriteLog("Channels               : ");
	if(IsProcessorFeaturePresent(16)) WriteLog(MSG_YES);
	else							  WriteLog(MSG_NO);
	WriteLog(MSG_NEWLINE);

	// Floating-point emulation.
	WriteLog("Soft. Float Emulation  : ");
	if(IsProcessorFeaturePresent(PF_FLOATING_POINT_EMULATED)) WriteLog(MSG_YES);
	else											          WriteLog(MSG_NO);
	WriteLog(MSG_NEWLINE);

	// DEP
	WriteLog("DEP Enabled            : ");
	if(IsProcessorFeaturePresent(12)) WriteLog(MSG_YES);
	else						      WriteLog(MSG_NO);
	WriteLog(MSG_NEWLINE);

	// PAE
	WriteLog("PAE Enabled            : ");
	if(IsProcessorFeaturePresent(PF_PAE_ENABLED)) WriteLog(MSG_YES);
	else									      WriteLog(MSG_NO);
	WriteLog(MSG_NEWLINE);


	//
	// Begin application log.
	//
	WriteLog("\r\n\r\n\r\n------------------------------------------------------------------\r\n");
	WriteLog("                          Process Snapshot                        \r\n");
	WriteLog("------------------------------------------------------------------\r\n");

	// Take a process snapshot.
	hProcSnap = CreateToolhelp32Snapshot(TH32CS_SNAPALL, NULL);

	if(hProcSnap != INVALID_HANDLE_VALUE)
	{
		ProcInfo.dwSize = sizeof(PROCESSENTRY32);

		if(Process32First(hProcSnap, &ProcInfo))
		{
			do
			{
				// Log process and associated module information.
				WriteLog(ProcInfo.szExeFile);
				WriteLog(MSG_NEWLINE);

				ProcInfo.dwSize = sizeof(PROCESSENTRY32);
			} while(Process32Next(hProcSnap, &ProcInfo));
		}

		CloseHandle(hProcSnap);
	}



	//
	// Begin application log.
	//
	WriteLog("\r\n\r\n\r\n------------------------------------------------------------------\r\n");
	WriteLog("                          Application Log                         \r\n");
	WriteLog("------------------------------------------------------------------\r\n");
}



//--------------------------------------------------------------------------
// Void CloseLog()
// ===============
//
// Description:
//   Closes the application log, disabling its use.
//
//
// Parameters:
//   None.
//
//
// Return Value:
//   None.
//
//
Void __stdcall CloseLog()
{
	//
	// Write any pending data to the log.
	//
	FlushFileBuffers(hLog);


	//
	// Close the log.
	//
	CloseHandle(hLog);
}
