/*
 * Utility.cpp
 * ===========
 * Copyright (C) Matthew J. W.
 * All rights reserved.
 * Version 1.0
 *
 *
 * Abstract:
 *   This module covers routines which are of a very general nature, and could
 *   not be categorized any other way.
 *
 *
 *******************************************************************************************************************************************************/
#include "BnetDiagTool.h"



////////////////////////////////////////////////////////////////////////////
//
// Math Functions
//
////////////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------------
// Dword PowerOf(Dword, Dword)
// ===========================
//
// Description:
//   Calculates the power of a base raised to an exponent. 
//
//
// Parameters:
//   [Dword] [dwBase]     - The base being raised.
//   [Dword] [dwExponent] - Power the base is being raised to.
//
//
// Return Value:
//   Returns the base raised to the given power.
//
//
Dword __fastcall PowerOf(Dword dwBase, Dword dwExponent)
{
	__asm
	{
		/**
		 *
		 * ECX - Base.
		 * EBX - Exponent.
		 * EAX - Power of the base to the exponent.
		 *
		 **/

		// Since MUL uses EDX, we must use EBX for the exponent instead.
		mov ebx, edx

		// Initialize the raised power to 1.
		mov eax, 1

		// Raise the base to the exponent.
		REITERATE:
			// Check if we've finished.
			cmp ebx, 0
			jz BREAK

			// Raise the base by 1.
			mul ecx

			// Remember how many times we've raised the base.
			dec ebx
			jmp REITERATE
		BREAK:
		EXIT:
	}
}



////////////////////////////////////////////////////////////////////////////
//
// Memory Functions
//
////////////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------------
// Word Endian16(Word)
// ===================
//
// Description:
//   Reverses the byte-order of a 16-bit value.
//
//
// Parameters:
//   [Word] [w] - The value of whose byte-order is to be reversed.
//
//
// Return Value:
//   Returns the given 16-bit value in the oppisite byte-order.
//
//
Word __fastcall Endian16(const Word w)
{
	__asm
	{
		// Reverse the endian (byte order) of the given 16-bit value.
		rol cx, 8

		// Return the converted value.
		mov ax, cx
	}
}



//--------------------------------------------------------------------------
// Dword Endian16(Dword)
// =====================
//
// Description:
//   Reverses the byte-order of a 32-bit value.
//
//
// Parameters:
//   [Dword] [dw] - The value of whose byte-order is to be reversed.
//
//
// Return Value:
//   Returns the given 32-bit value in the oppisite byte-order.
//
//
Dword __fastcall Endian32(const Dword dw)
{
	__asm
	{
		// Reverse the endian (byte order) of the given 32-bit value.
		rol ecx, 16

		// Return the converted value.
		mov eax, ecx
	}
}
	


////////////////////////////////////////////////////////////////////////////
//
// String & Character Functions
//
////////////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------------
// Dword GetStringLength(Char*)
// ============================
//
// Description:
//   Retrieves the length of an ASCII null-terminated string of characters.
//
//
// Parameters:
//   [Char*] [pstr] - The string to retrieve the length of.
//
//
// Return Value:
//   Returns the length of the given string.
//
//
Dword __fastcall GetStringLength(const Char* pstr)
{
	__asm
	{
		/**
		 *
		 * EAX - Maintains a count of the length of the string.
		 * ECX - Points to the current string element.
		 *
         **/

		// Initialize the length count to zero.
		mov eax, 0

		// Count the number of non-null elements of the string.
		REITERATE:
			// If the end of the string hasn't been reached, move on to the next
		    // element, and increment the count accordingly.
			cmp [ecx], 0
			jz BREAK
			inc eax
			inc ecx
			jmp REITERATE
		BREAK:
	}
}



//--------------------------------------------------------------------------
// Dword IsAlpha(Char)
// ===================
//
// Description:
//   Determines whether or not a character is part of the alphabet.
//
//
// Parameters:
//   [Char] [c] - The character being queried.
//
//
// Return Value:
//   (2) The character is part of the alphabet, and is lower-case.
//   (1) The character is part of the alphabet, and is upper-case.
//   (0) The character is not part of the alphabet.
//
//
Dword __fastcall IsAlpha(const Char c)
{
	__asm
	{
		/**
		 *
		 * ECX - The character being queried.
		 *
		 **/

		// The query is not alpha until proven otherwise.
		mov eax, 0

		// Check if the query is below the range of possible alpha characters.
		cmp cl, 65
		jb EXIT

		// Check if the query is a upper-case alpha character.
		cmp cl, 90
		jbe IS_ALPHA_UPPER

		// Check if the query is above the range of possible alpha characters.
		cmp cl, 122
		ja EXIT

		// Check if the query is a lower-case alpha character.
		cmp cl, 97
		jae IS_ALPHA_LOWER

		// The query is not in the range of A-Z or a-z, therefor it's not an
		// alpha character.
		jmp EXIT

		IS_ALPHA_UPPER:
		mov eax, 1
		jmp EXIT

		IS_ALPHA_LOWER:
		mov eax, 2

		EXIT:
	}
}



//--------------------------------------------------------------------------
// Dword CompareString(Char*, Char*)
// =================================
//
// Description:
//   Compares two ASCII null-terminated strings against one-another.
//
//
// Parameters:
//   [Char*] [pstrLeft]  - First string for comparison.
//   [Char*] [pstrRight] - Second string for comparison.
//
//
// Return Value:
//   If the strings match, a non-zero value is returned. Elsewise, if the strings
//   don't match zero is returned.
//
//
//
Bool __fastcall CompareString(const Char* pstrLeft, const Char* pstrRight)
{
	__asm
	{
		/**
		 *
		 * ECX - Points to the current element of the left-hand string being compared.
		 * EDX - Points to the current element of the right-hand string being compared.
		 *
		 **/

		/**
		 *
		 * If the string's aren't of the same length, then they certainly don't match.
		 *
		 **/
		
		// Retrieve the length of both the left and right strings, the results
		// being stored in EAX and EBX.
		mov esi, ecx
		call GetStringLength
		mov ebx, eax
		mov ecx, edx
		call GetStringLength

		// Compare the length of the two strings, if they don't match then we
		// can claim they don't match.
		cmp ebx, eax
		jnz EXIT_NO_MATCH

		// Compare the corrosponding elements of each string against one-another.
		REITERATE:
			// Check if we've reached the end of the strings. This would indicate
			// the strings match.
			cmp [edx], 0
			jnz CONTINUE
			// The strings match.
			mov eax, 1
			jmp EXIT_MATCH

			CONTINUE:
			// Compare the elements, and if they don't match then obviously the
			// strings are _not_ equal.
			mov bl, [esi]
			cmp bl, [edx]
			jnz EXIT_NO_MATCH

			// Move on to the next element of each string.
			inc edx
			inc esi

			jmp REITERATE
		BREAK:
		EXIT_NO_MATCH:
		// The strings don't match.
		mov eax, 0
		
		EXIT_MATCH:	
	}
}



//--------------------------------------------------------------------------
// Dword CompareStringNoCase(Char*, Char*)
// =======================================
//
// Description:
//   Compares two ASCII null-terminated strings against one-another, ignoring
//   the case of alpha characters.
//
//
// Parameters:
//   [Char*] [pstrLeft]  - First string for comparison.
//   [Char*] [pstrRight] - Second string for comparison.
//
//
// Return Value:
//   If the strings match, a non-zero value is returned. Elsewise, if the strings
//   don't match zero is returned.
//
//
//
Bool __fastcall CompareStringNoCase(const Char* pstrLeft, const Char* pstrRight)
{
	__asm
	{
		/**
		 *
		 * ECX - Points to the current element of the left-hand string being compared.
		 * EDX - Points to the current element of the right-hand string being compared.
		 *
		 **/

		/**
		 *
		 * If the string's aren't of the same length, then they certainly don't match.
		 *
		 **/
		
		// Retrieve the length of both the left and right strings, the results
		// being stored in EAX and EBX.
		mov esi, ecx
		call GetStringLength
		mov ebx, eax
		mov ecx, edx
		call GetStringLength

		// Compare the length of the two strings, if they don't match then we
		// can claim they don't match.
		cmp ebx, eax
		jnz EXIT_NO_MATCH

		// Compare the corrosponding elements of each string against one-another,
		// ignoring their possible case differences.
		REITERATE:
			// Check if we've reached the end of the strings. This would indicate
			// the strings match.
			cmp [edx], 0
			jnz CONTINUE
			// The strings match.
			mov eax, 1
			jmp EXIT_MATCH

			CONTINUE:
			// Compare the elements, and if they don't match then obviously the
			// strings are _not_ equal.
			mov bl, [esi]

			COMPARE:
			cmp bl, [edx]
			jz CONTINUE2

			// See if we're able to switch the case of one of the characters.
			mov cl, bl
			call IsAlpha
			cmp eax, 0
			jz EXIT_NO_MATCH

			// If the character is upper-case, switch it to lower-case.
			cmp eax, 1
			jnz NOT_UPPER
			add bl, 32
			jmp COMPARE_CASE_SWITCH

			NOT_UPPER:
			sub bl, 32

			COMPARE_CASE_SWITCH:
			// Compare the characters again, after the case-switch.
			cmp bl, [edx]
			jnz EXIT_NO_MATCH

			CONTINUE2:
			// Move on to the next element of each string.
			inc edx
			inc esi

			jmp REITERATE
		BREAK:
		EXIT_NO_MATCH:
		// The strings don't match.
		mov eax, 0
		
		EXIT_MATCH:	
	}
}



//--------------------------------------------------------------------------
// Byte CharToInt(Char)
// ====================
//
// Description:
//   Converts an ASCII character representing an integer, to an actualy integer
//   in it's binary form.
//
//
// Parameters:
//   [Char] [c] - The character being converted.
//
//
// Return Value:
//   Returns the binary form of the digit being represented by the given character.
//
//
Byte __fastcall CharToInt(const Char c)
{
	__asm
	{
		/**
		 *
		 * CL - The character being converted to binary.
		 *
		 **/

		// We'll use EAX now.
		movzx eax, cl

		// Ensure the character is not below the minimum range of decimal digits.
		cmp al, 48
		jb EXIT

		// Check if the character is not part of the decimal system.
		cmp al, 57
		ja HEX
		// The character is a decimal digit. Convert it.
		sub al, 48
		jmp EXIT

		HEX:
		// The character is a hexadecimal digit. Convert it.
		sub al, 55

		EXIT:
	}
}



Char __fastcall IntToChar(const Byte i)
{
	__asm
	{
		/**
		 *
		 * CL - The integer value being converted.
		 *
		 **/

		// We'll use EAX now.
		movzx eax, cl

		// Check if the integer represents a hexadecimal digit. 
		cmp al, 9
		ja HEX
		// The integer is in the decimal range (0-9), convert it.
		add al, 48
		jmp EXIT

		HEX:
		// The integer is in the hexadecimal range (A-F), convert it.
		add al, 55

		EXIT:
	}
}



//--------------------------------------------------------------------------
// Dword StrToInt(Char*, Dword)
// ============================
//
// Description:
//   Converts a string of characters representing an integer to an actual
//   binary integer.
//
//
// Parameters:
//   [Char*] [pstr]   - The string to be converted.
//   [Dword] [dwBase] - The base number system the given string is in.
//
// Return Value:
//   Returns the binary form of the given string.
//
//
// Remarks:
//   The 'dwBase' parameters must be greater than 1. If it isn't, StrToInt()'s
//   behaviour is undefined.
//
//
Dword __fastcall StrToInt(const Char* pstr, Dword dwBase)
{
	Dword dwResult = 0;


	__asm
	{
		/**
		 *
		 * ESI - Current digit position being processed.
		 * EDI - Points to the current character being processed.
		 * EBX - Base system being used.
		 *
		 **/

		// Swap parameters into different registers so we can interface with
		// other routines properely.
		mov edi, ecx
		mov ebx, edx

		// Retrieve the length of the string, so we know the greatest positional
		// value it's using.
		call GetStringLength
		mov esi, eax

		// Check if the string is empty, in which case the result is always going
		// to be zero.
		cmp esi, 0
		jz EXIT
		dec esi
		
		// Convert the string into the integer it's representing.
		REITERATE:
			// Check if we've reached the end of the string.
			cmp esi, 4294967295
			jz BREAK

			// Calculate the positional value of the current digit.
			mov ecx, ebx // 'dwBase'
			mov edx, esi // 'dwExponent'
			call PowerOf
			mov edx, eax

			// Convert the current character to it's proper integer form.
			mov cl, [edi]
			call CharToInt

			// Calculate the position's contribution.
			mov ecx, edx
			mul ecx

			// Add the positions contribution to the final result.
			add [dwResult], eax

			// Move through the string.
			dec esi
			inc edi

			jmp REITERATE
		BREAK:

		EXIT:
		// Return the proper value.
		mov eax, [dwResult]
	}
}



//--------------------------------------------------------------------------
// Void IntToStr(Dword, Dword, Char*)
// ==================================
//
// Description:
//   Converts an integer into a string of characters appropiately representing
//   it.
//
//
// Parameters:
//   [Dword] [dw]         - The integer value being converted.
//   [Dword] [dwBase]     - The base system the output string should be in.
//   [Char*] [pstrOutput] - Pointer to a buffer which will receive the resulting string.
//
// Return Value:
//   Returns the binary form of the given string.
//
//
// Remarks:
//   None.
//
//
Void __fastcall IntToStr(Dword dw, Dword dwBase, Char* pstrOutput)
{
	__asm
	{
		/**
		 *
		 * ESI - Index of the current digit being processed.
		 * EDI - Base system.
		 * EBX - The integer value.
		 *
		 **/

		// Check if the integer is zero, so we can exit quick.
		cmp ecx, 0
		jnz CONTINUE

		// Zero is always zero.
		mov ebx, pstrOutput
		mov [ebx], 48 // '0'
		jmp EXIT

		CONTINUE:
		// Prepare alternate parameter storage.
		mov ebx, ecx
		mov edi, edx

		// Begin at the smallest digit index.
		mov esi, 0

		// Retrieve the greatest positional value being used by the integer.
		REITERATE:
			// Calculate the current positional value.
			mov ecx, edi
			mov edx, esi
			call PowerOf

			// Put the positional value on the stack for later use, so we
			// don't have to re-calculate it.
			push eax

			// Check if we've reached a positional value which exceeds our
			// 32-bit boundaries. Which means no matter what, we've reached
			// a positional value not needed by the integer.
			cmp edx, 0
			jnz BREAK

			// Check if we've found a positional value that is not needed
			// by the integer.
			cmp eax, ebx
			ja BREAK

			// Move on to the next digit index, and thus positional value.
			inc esi

			jmp REITERATE
		BREAK:

		// Remove the unused positional value from the stack.
		pop eax

		// This will prevent integer-overflows.
		mov edx, 0

		// Generate the string representation of the integer.
		REITERATE2:
			// Check if we've finished converting the whole integer.
			cmp esi, 0
			jz BREAK2

			// Retrieve the next positional value being processed.
			pop ecx

			// Calculate the number of times the positional value is being
			// used. Hence, the digit which would be residing under it in
			// the number.
			mov eax, ebx
			div ecx
			mov edi, eax

			// Calculate the contribution of this positional value, and remove
			// it from the integer.
			mul ecx
			sub ebx, eax

			// Convert the digit into it's character form.
			mov ecx, edi
			call IntToChar

			// Copy the character into the output string.
			mov ecx, pstrOutput
			mov [ecx], al


			// Move on to the next positional value, and the next element in the
			// output string.
			dec esi
			inc [pstrOutput]

			jmp REITERATE2
		BREAK2:

		EXIT:
	}
}